"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateServiceMethodTemplate = exports.TemplateServiceMethodSchema = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const core_1 = require("@angular-devkit/core");
const ts = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const change_1 = require("@schematics/angular/utility/change");
const updateFile_1 = require("../utils/updateFile");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const utils_1 = require("../utils/utils");
class TemplateServiceMethodSchema {
}
exports.TemplateServiceMethodSchema = TemplateServiceMethodSchema;
function generateServiceMethodTemplate(options) {
    return () => {
        var _a, _b;
        (_a = options.scrollName) !== null && _a !== void 0 ? _a : (options.scrollName = "");
        (_b = options.envFilePath) !== null && _b !== void 0 ? _b : (options.envFilePath = utils_1.DevEnvFile);
        let serviceMethodFilePath = (0, core_1.normalize)(`/${options.path}`);
        const templateSource = (0, schematics_1.apply)((0, schematics_1.url)('./files/' + options.transformationType), [
            (0, schematics_1.applyTemplates)(Object.assign({ capitalize: schematics_1.strings.capitalize }, options)),
            (0, schematics_1.move)(serviceMethodFilePath),
        ]);
        return (0, schematics_1.chain)([
            (0, schematics_1.mergeWith)(templateSource),
            updateServiceFile(options)
        ]);
    };
}
exports.generateServiceMethodTemplate = generateServiceMethodTemplate;
let updateServiceFile = (options) => {
    return (tree) => {
        var _a;
        let targetFileName = options.serviceName + ".service.ts";
        let targetFilePath = (0, core_1.normalize)(`${options.path}/${targetFileName}`);
        let targetFile = tree.read(targetFilePath);
        if (!targetFile) {
            throw new schematics_1.SchematicsException(`${targetFilePath} not found make sure the command is run in the directory of the service file`);
        }
        let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
        let scrollingNameString = (0, utils_1.createStringObject)((_a = options.scrollName) !== null && _a !== void 0 ? _a : "");
        let serviceString = (0, utils_1.createStringObject)(options.serviceName + "Service", "Service");
        let serviceMethodString = (0, utils_1.createStringObject)(options.name, "");
        let serviceClassVar = sourceFile.statements
            .filter(ts.isClassDeclaration)
            .find((v) => {
            var _a;
            return ((_a = v.name) === null || _a === void 0 ? void 0 : _a.getText()) === serviceString.classify(false);
        });
        addServiceMethodToServiceClass(options, serviceString, serviceMethodString, serviceClassVar, targetFilePath, tree, scrollingNameString);
        addNeededImportsToServiceClass(sourceFile, serviceMethodString, options, targetFilePath, tree);
        updateConstructor(options, sourceFile, targetFilePath, tree, serviceClassVar);
        updateEnvFile(options, tree, serviceString);
        updateSpecFile(options, tree, serviceMethodString, serviceString);
        // let transformFileName = options.name+".ts"
        // let transformFilePath = normalize(`${options.path}/${transformFileName}`)
        // sourceFile = wmlCreateSourceFile(targetFilePath, tree);
        // console.log(sourceFile.getFullText())
        return tree;
    };
};
function updateConstructor(options, sourceFile, targetFilePath, tree, serviceClassVar) {
    if (options.transformationType === "scrolling") {
        sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
        serviceClassVar = (0, utils_1.getServiceClassVar)(sourceFile);
        (0, utils_1.performDependencyInjection)(targetFilePath, tree, `public baseService:BaseService`, serviceClassVar);
    }
    return { sourceFile, serviceClassVar };
}
function addNeededImportsToServiceClass(sourceFile, serviceMethodString, options, targetFilePath, tree) {
    let importsToAdd = [
        {
            packageName: `./${options.name}`,
            importStatement: `import { ${serviceMethodString.capitalize(false)}UIRequestBody,${serviceMethodString.capitalize(false)}UIResponseBody,${options.name}Load, ${options.name}Success } from './${options.name}';`,
            importMembers: [
                `${serviceMethodString.capitalize(false)}UIRequestBody`,
                `${serviceMethodString.capitalize(false)}UIResponseBody`,
                `${options.name}Load`, `${options.name}Success`
            ]
        }
    ];
    if (options.transformationType === "scrolling") {
        importsToAdd.push({
            packageName: '@core/utility/scroll-utils',
            importStatement: `import { listEntitiesFail, ScrollPageParams } from '@core/utility/scroll-utils';`,
            importMembers: [
                'listEntitiesFail', 'ScrollPageParams'
            ]
        });
        importsToAdd.push({
            packageName: 'rxjs',
            importStatement: `import { retry, delay } from 'rxjs';`,
            importMembers: ["retry", "delay"]
        });
        importsToAdd.push({
            packageName: '@core/base/base.service',
            importStatement: `import { BaseService } from '@core/base/base.service';`,
            importMembers: ["BaseService"]
        });
    }
    (0, utils_1.addImportStatements)(sourceFile, targetFilePath, tree, importsToAdd);
}
function addServiceMethodToServiceClass(options, serviceString, serviceMethodString, serviceClassVar, targetFilePath, tree, scrollingNameString) {
    let serviceMethodStringObj = `${serviceMethodString.orig} = (uiBody = new ${serviceMethodString.capitalize(false)}UIRequestBody(),raw =false)=>{
    return iif(
      ()=>ENV.${serviceString.camelCase(false)}.${options.name}.automate,
      of(new ${serviceMethodString.capitalize(false)}UIResponseBody()),
      ${options.name}Load(uiBody)
        .pipe(
          concatMap((apiBody)=>{
            return this.http
              .post(ENV.${serviceString.camelCase(false)}.${options.name}.url(),apiBody)
              .pipe(raw ? tap() : map(${options.name}Success))
          })
        )
      )
    }`;
    if (options.transformationType === "scrolling") {
        serviceMethodStringObj = `${scrollingNameString.orig}ScrollPageUIRequestBody = new ${serviceMethodString.classify(false)}UIRequestBody({pageSize:10})
${scrollingNameString.orig}ScrollPageObj:ScrollPageParams<any>

${serviceMethodString.orig} = (uiBody = new ${serviceMethodString.capitalize(false)}UIRequestBody(),scrollPage?:ScrollPageParams,raw =false)=>{
  this.baseService.openOverlayLoading()
  return iif(
    ()=>ENV.${serviceString.camelCase(false)}.${options.name}.automate,
    of(new ${serviceMethodString.capitalize(false)}UIResponseBody())
    .pipe(
      delay(300)
    ),
    ${options.name}Load(uiBody)
      .pipe(
        concatMap((apiBody)=>{
          return this.http
            .post(ENV.${serviceString.camelCase(false)}.${options.name}.url(),apiBody)
            .pipe(
              retry(3),
              tap({
                next:(apiBody:any)=>{
                  this.baseService.notifyUserThatThereAreNoRecords(apiBody, uiBody);
                },
                error:()=>{
                  listEntitiesFail(uiBody)
                }
              }),
              raw ? tap() : map(${options.name}Success(scrollPage,uiBody))
            )
        })
      )
    )
    .pipe(
      this.baseService.closeOverlayLoadingViaRxjsFinalize
    )
  }`;
    }
    let occurences = serviceClassVar.members.length;
    let text = serviceClassVar.getFullText();
    let insertPos = serviceClassVar.members.pos;
    let final = serviceMethodStringObj;
    if (occurences > 0) {
        let lastMember = [
            ...serviceClassVar.members
        ].pop();
        let indentation = text.match(/\r?\n(\r?)\s*/) || [];
        final = `${indentation[0] || ' '}${serviceMethodStringObj}`;
        insertPos = lastMember.end;
    }
    let changes = [new change_1.InsertChange(targetFilePath, insertPos, final)];
    (0, updateFile_1.wmlUpdateFile)(tree, targetFilePath, changes);
}
function updateEnvFile(options, tree, serviceString) {
    var _a, _b;
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(options.envFilePath, tree);
    let devEnv = (_a = sourceFile === null || sourceFile === void 0 ? void 0 : sourceFile.statements) === null || _a === void 0 ? void 0 : _a.filter(ts.isClassDeclaration).find((v) => {
        var _a;
        return ((_a = v.name) === null || _a === void 0 ? void 0 : _a.getText()) === 'DevEnv';
    });
    let methodString = `${options.name}:new WMLEndpoint({
        url:()=> this.backendDomain0 + "${options.apiRoute}",
      })`;
    let envClassProp = devEnv.members.find((prop) => {
        var _a;
        return ((_a = prop.name) === null || _a === void 0 ? void 0 : _a.getText()) === serviceString.camelCase(false);
    });
    if (!envClassProp) {
        let lastProperty = (_b = devEnv.members
            .filter(ts.isPropertyDeclaration).at(-1)) !== null && _b !== void 0 ? _b : (0, utils_1.wmlFindChildNode)(devEnv.members, (prop) => prop.kind === ts.SyntaxKind.OpenBraceToken, "This DevEnv class is malformed I am looking for an openBrace", false, false);
        let snippet = `\n${serviceString.camelCase(false)} = {
    ${methodString},
    }
`;
        (0, utils_1.prepareForUpdate)(snippet, options.envFilePath, tree, lastProperty.end, sourceFile.getFullText());
    }
    else {
        let serviceClassObj = (0, ast_utils_1.findNodes)(envClassProp, ts.SyntaxKind.ObjectLiteralExpression, 1)[0];
        let occurrencesCount = serviceClassObj.properties.length;
        let text = serviceClassObj.getFullText();
        let insertPos = serviceClassObj.properties.pos;
        let finalVar = methodString;
        if (occurrencesCount > 0) {
            let lastEndpointLiteral = [
                ...serviceClassObj.properties,
            ].pop();
            let indentation = (0, utils_1.getIndentation)(text);
            let endpointText = `${indentation[0] || ' '}${methodString}`;
            insertPos = lastEndpointLiteral.end;
            finalVar = `,${endpointText}`;
        }
        let changes = [new change_1.InsertChange(options.envFilePath, insertPos, finalVar)];
        (0, updateFile_1.wmlUpdateFile)(tree, options.envFilePath, changes);
    }
}
function updateSpecFile(options, tree, serviceMethodString, serviceString) {
    let targetFileName = options.serviceName + ".service.spec.ts";
    let targetFilePath = (0, core_1.normalize)(`${options.path}/${targetFileName}`);
    let targetFile = tree.read(targetFilePath);
    if (!targetFile) {
        throw new schematics_1.SchematicsException(`${targetFilePath} not found make sure the command is run in the directory of the service file`);
    }
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    addNeededImportsToServiceTestFile(sourceFile, serviceMethodString, options, targetFilePath, tree);
    sourceFile = addRequireBlockToSpecFile(targetFilePath, tree, options);
    sourceFile = addMockAPIResultToSpecFile(targetFilePath, tree, serviceMethodString, serviceString);
    sourceFile = addSpiesToSpecFile(targetFilePath, tree, serviceMethodString, serviceString);
    sourceFile = addDescribeBlockToServiceSpecFile(targetFilePath, tree, serviceMethodString, serviceString);
    // sourceFile = wmlCreateSourceFile(targetFilePath, tree);
    // console.log(sourceFile.getFullText())
    // throw new SchematicsException("stop here")
}
function addNeededImportsToServiceTestFile(sourceFile, serviceMethodString, options, targetFilePath, tree) {
    let importsToAdd = [
        {
            packageName: `./${options.name}`,
            importStatement: `import { ${serviceMethodString.capitalize(false)}APIRequestBody,${serviceMethodString.capitalize(false)}APIResponseBody,${serviceMethodString.capitalize(false)}UIRequestBody,${serviceMethodString.capitalize(false)}UIResponseBody   } from './${options.name}';`,
            importMembers: [
                `${serviceMethodString.capitalize(false)}APIResponseBody`,
                `${serviceMethodString.capitalize(false)}APIRequestBody`,
                `${serviceMethodString.capitalize(false)}UIRequestBody`,
                `${serviceMethodString.capitalize(false)}UIResponseBody`,
            ]
        },
        {
            packageName: 'rxjs',
            importStatement: `import { of } from 'rxjs';`,
            importMembers: ["of"]
        },
        {
            packageName: '@env/environment',
            importStatement: `import { ENV } from '@env/environment';`,
            importMembers: ["ENV"]
        },
        {
            packageName: '@angular/core/testing',
            importStatement: `import { discardPeriodicTasks, fakeAsync, flush } from '@env/environment';`,
            importMembers: ["discardPeriodicTasks", "fakeAsync", "flush"]
        }
    ];
    (0, utils_1.addImportStatements)(sourceFile, targetFilePath, tree, importsToAdd);
}
function addRequireBlockToSpecFile(targetFilePath, tree, options) {
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let target;
    let requireBlocks = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
        node: sourceFile,
        kind: (node) => {
            let isKind = node.kind === ts.SyntaxKind.VariableStatement;
            let regexpResult = RegExp(/(.+?)require\(.+?\)(|.+?)/).exec(node.getFullText());
            let stringContainingRequireIsOneLine = Boolean(regexpResult);
            return isKind && stringContainingRequireIsOneLine;
        },
    }));
    let lastRequireBlock = requireBlocks.at(-1);
    let snippet = `let ${options.name} = require('./${options.name}');`;
    let spacingPrefix = "\n";
    if (!lastRequireBlock) {
        snippet = "\n" + snippet;
        sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
        let statements = sourceFile.statements
            .filter(ts.isImportDeclaration);
        let lastMember = statements.at(-1);
        target = lastMember;
    }
    else {
        target = lastRequireBlock;
    }
    (0, utils_1.prepareForUpdate)(snippet, targetFilePath, tree, 
    // @ts-ignore
    target.end, spacingPrefix);
    return sourceFile;
}
function addMockAPIResultToSpecFile(targetFilePath, tree, serviceMethodString, serviceString) {
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let targets = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
        node: sourceFile,
        kind: (node) => {
            let isKind = node.kind === ts.SyntaxKind.VariableStatement;
            let regexpResult = RegExp(/(.+)Service(|;)/).exec(node.getFullText());
            let stringContainingRequireIsOneLine = Boolean(regexpResult);
            return isKind && stringContainingRequireIsOneLine;
        },
    }));
    let target;
    let lastServiceDecBlock = targets.at(-1);
    let snippet = `let ${serviceMethodString.camelCase(false)}LoadResult = new ${serviceMethodString.capitalize(false)}APIRequestBody();`;
    let spacingPrefix = "\n  ";
    let insertPos = 0;
    if (!lastServiceDecBlock) {
        snippet = "\n" + snippet;
        sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
        let targets = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
            node: sourceFile,
            kind: (node) => {
                let isKind = node.kind === ts.SyntaxKind.Block;
                let regexpResult = RegExp(/\{.+(beforeEach|describe|it).+/s).exec(node.getFullText());
                let stringContainingRequireIsOneLine = Boolean(regexpResult);
                let alsoFoundServiceName;
                if (regexpResult) {
                    let result;
                    if (node.parent) {
                        result = (0, utils_1.wmlFindChildNode)(node.parent, (node) => {
                            return node.getFullText().includes(serviceString.capitalize(false));
                        }, "", false, false);
                    }
                    alsoFoundServiceName = Boolean(result);
                }
                return isKind && stringContainingRequireIsOneLine && alsoFoundServiceName;
            },
        }));
        let target = targets.at(0);
        console.log(target === null || target === void 0 ? void 0 : target.getFullText());
        // @ts-ignore
        insertPos = target.pos + 2;
    }
    else {
        target = lastServiceDecBlock;
        insertPos = target.end;
    }
    (0, utils_1.prepareForUpdate)(snippet, targetFilePath, tree, 
    // @ts-ignore
    insertPos, spacingPrefix);
    return sourceFile;
}
function addSpiesToSpecFile(targetFilePath, tree, serviceMethodString, serviceString) {
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let startNode;
    let targets;
    if (!startNode) {
        targets = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
            node: sourceFile,
            kind: (node) => {
                let boolVal0 = node.kind === ts.SyntaxKind.ExpressionStatement;
                // if issues update this regex
                let regexpResult = RegExp(`(|.+)spyOn((|.+)LoadResult)(|;)`, "s").exec(node.getFullText());
                let boolVal1 = Boolean(regexpResult);
                let boolVal2;
                if (boolVal1) {
                    let binExpr0 = (0, utils_1.wmlFindChildNode)(node, (child) => {
                        return [ts.SyntaxKind.CallExpression].includes(child.kind);
                    }, "", false, false);
                    boolVal2 = Boolean(binExpr0);
                }
                return boolVal0 && boolVal1 && boolVal2;
            },
        }));
        startNode = targets.at(-1);
    }
    // console.log("0 ",startNode?.getFullText())
    if (!startNode) {
        targets = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
            node: sourceFile,
            kind: (node) => {
                let boolVal0 = node.kind === ts.SyntaxKind.ExpressionStatement;
                // if issues update this regex
                let regexpResult = RegExp(/(|.+)wmlTestUtils\s*.spyOnForES6Imports/s).exec(node.getFullText());
                let boolVal1 = Boolean(regexpResult);
                let boolVal2;
                if (boolVal1) {
                    let binExpr0 = (0, utils_1.wmlFindChildNode)(node, (child) => {
                        return [ts.SyntaxKind.BinaryExpression, ts.SyntaxKind.CallExpression].includes(child.kind);
                    }, "", false, false);
                    boolVal2 = Boolean(binExpr0);
                }
                return boolVal0 && boolVal1 && boolVal2;
            },
        }));
        startNode = targets.at(-1);
    }
    // console.log("1 ",startNode?.getFullText())
    if (!startNode) {
        targets = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
            node: sourceFile,
            kind: (node) => {
                let boolVal0 = node.kind === ts.SyntaxKind.ExpressionStatement;
                let regexpResult = RegExp(/(|.+)spyOn((|.+)(cpnt|service)(| +),(| +)('|")(|.+)('|")(|.+)).and.callThrough\(\)(|;)/s).exec(node.getFullText());
                let boolVal1 = Boolean(regexpResult);
                let boolVal2;
                if (boolVal1) {
                    let binExpr0 = (0, utils_1.wmlFindChildNode)(node, ts.SyntaxKind.CallExpression, "", false, false);
                    boolVal2 = Boolean(binExpr0);
                }
                return boolVal0 && boolVal1 && boolVal2;
            },
        }));
        startNode = targets.at(-1);
    }
    // console.log("2 ",startNode?.getFullText())
    if (!startNode) {
        targets = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
            node: sourceFile,
            kind: (node) => {
                let boolVal0 = node.kind === ts.SyntaxKind.ExpressionStatement;
                // let regexpResult = RegExp(/spyOn(\s*(cpnt|service)\.http\s*,\s*('|").*('|")\s*).*/).exec(node.getFullText());
                let regexpResult = RegExp(/spyOn\((cpnt|service).http\s*,.*\);?/).exec(node.getFullText());
                let boolVal1 = Boolean(regexpResult);
                let boolVal2;
                if (boolVal1) {
                    let binExpr0 = (0, utils_1.wmlFindChildNode)(node, ts.SyntaxKind.CallExpression, "", false, false);
                    boolVal2 = Boolean(binExpr0);
                }
                return boolVal0 && boolVal1 && boolVal2;
            },
        }));
        startNode = targets.at(-1);
    }
    // console.log("3 ",startNode?.getFullText())
    if (!startNode) {
        targets = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
            node: sourceFile,
            kind: (node) => {
                let isKind = node.kind === ts.SyntaxKind.ExpressionStatement;
                let regexpResult = RegExp(/(|.+)utilService(|.+)=(|.+)TestBed.inject((|.+)UtilityService(|.+))(|.+)/s).exec(node.getFullText());
                let stringContainingConfigure = Boolean(regexpResult);
                let isBinExpr0;
                if (stringContainingConfigure) {
                    let binExpr0 = (0, utils_1.wmlFindChildNode)(node, ts.SyntaxKind.BinaryExpression, "", false, false);
                    isBinExpr0 = Boolean(binExpr0);
                }
                return isKind && stringContainingConfigure && isBinExpr0;
            },
        }));
        startNode = targets.at(0);
    }
    // console.log("4 ",startNode?.getFullText())
    if (!startNode) {
        targets = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
            node: sourceFile,
            kind: (node) => {
                let isKind = node.kind === ts.SyntaxKind.ExpressionStatement;
                let regexpResult = RegExp(/(|.+)wmlTestUtils.configureTestingModuleForServices\(.+\)(|.+)/).exec(node.getFullText());
                let stringContainingConfigure = Boolean(regexpResult);
                let isBinExpr0;
                if (stringContainingConfigure) {
                    let binExpr0 = (0, utils_1.wmlFindChildNode)(node, ts.SyntaxKind.BinaryExpression, "", false, false);
                    isBinExpr0 = Boolean(binExpr0);
                }
                return isKind && stringContainingConfigure && isBinExpr0;
            },
        }));
        startNode = targets.at(0);
    }
    // console.log("5 ",startNode?.getFullText())
    if (!startNode) {
        throw new schematics_1.SchematicsException(`service must be instantiated like so
    and for the beginners rmbr to remove the arrows as they are placeholders and dont belong in javascript!!

    service = wmlTestUtils.configureTestingModuleForServices(>>Service Name <<);
    `);
    }
    let snippet = `\nspyOn(${serviceMethodString.orig}, '${serviceMethodString.orig}Success')
  .and.callThrough();
spyOn(${serviceMethodString.orig}, '${serviceMethodString.orig}Load')
  .and.callFake(()=>of(${serviceMethodString.orig}LoadResult));
  `;
    let spacingPrefix = "\n  ";
    let insertPos = startNode.end;
    (0, utils_1.prepareForUpdate)(snippet, targetFilePath, tree, 
    // @ts-ignore
    insertPos, spacingPrefix);
    return sourceFile;
}
function addDescribeBlockToServiceSpecFile(targetFilePath, tree, serviceMethodString, serviceString) {
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let startNode;
    let targets;
    if (!startNode) {
        let targets = (0, utils_1.findNodeTypeByRecursion)(new utils_1.FindNodeTypeByRecursionParams({
            node: sourceFile,
            printChildInfo: false,
            kind: (node, level) => {
                let boolVal0 = node.kind === ts.SyntaxKind.ExpressionStatement;
                // if issues update this regex
                let regexpResult = RegExp(`describe.*['"]${serviceString.capitalize(false)}['"]`, "s").exec(node.getFullText());
                let boolVal1 = Boolean(regexpResult);
                let boolVal2;
                return boolVal0 && boolVal1;
            },
        }));
        let callExpr0 = (0, utils_1.wmlFindChildNode)(targets[0], ts.SyntaxKind.CallExpression);
        let syntaxList0 = (0, utils_1.wmlFindChildNode)(callExpr0, ts.SyntaxKind.SyntaxList);
        let arrowFn0 = (0, utils_1.wmlFindChildNode)(syntaxList0, ts.SyntaxKind.ArrowFunction, "");
        let block0 = (0, utils_1.wmlFindChildNode)(arrowFn0, ts.SyntaxKind.Block, "");
        let syntaxlist1 = (0, utils_1.wmlFindChildNode)(block0, ts.SyntaxKind.SyntaxList);
        startNode = (0, utils_1.wmlFindChildNode)(syntaxlist1, (node) => {
            return Boolean(RegExp(/describe.*['"].*['"]|beforeEach.*(.*)/).exec(node.getFullText())) && node.kind === ts.SyntaxKind.ExpressionStatement;
        }, "", false, false, "filter").at(-1);
    }
    // targets.forEach((x)=>{
    //   getTsMemberInfo(x)
    // })
    if (!startNode) {
        throw new schematics_1.SchematicsException(`the describe block may be missing
    please ensure at a minium the spec file contains



    describe('>>Service Name <<', () => {

    });
    if not

    then at least a beforeEach should be found to reset each test for the service like this

    describe('>>Service Name <<', () => {
      resetImports()
      resetProviders(>>Service Name <<)
      cpnt = wmlTestUtils.configureTestingModuleForServices(>>Service Name <<)
      utilService =TestBed.inject(UtilityService)
      spyOn(cpnt.http,"get").and.callThrough()
      spyOn(cpnt.http,"post").and.callThrough()
    });
    ${utils_1.WMLschematicMsgs.removePlacehoders0}
    if not contact support if the issue persists
    `);
    }
    let snippet = `\n  describe("${serviceMethodString.orig}",()=>{
    describe("when ${serviceMethodString.orig}.automate === false",()=>{
      it(\` when called |
      under normal conditions |
      does the required action \`,fakeAsync(()=>{
       // arrange
       ENV.${serviceString.camelCase(false)}.${serviceMethodString.orig}.automate = false
       // @ts-ignore
       cpnt.http.post = ()=>of(new ${serviceMethodString.capitalize(false)}APIResponseBody())
       spyOn(cpnt.http,"post").and.callThrough()
       let uiBody = new ${serviceMethodString.capitalize(false)}UIRequestBody()


      let successFn = (val?)=>{
        // assert
        expect(${serviceMethodString.orig}.${serviceMethodString.orig}Success).toHaveBeenCalled()
        discardPeriodicTasks()
      }

      // act
      cpnt.${serviceMethodString.orig}(uiBody)
      .subscribe({next:successFn,complete:successFn})
      expect(${serviceMethodString.orig}.${serviceMethodString.orig}Load).toHaveBeenCalledWith(uiBody)
      flush()
      expect(cpnt.http.post)
      .toHaveBeenCalledWith(ENV.${serviceString.camelCase(false)}.${serviceMethodString.orig}.url(),${serviceMethodString.orig}LoadResult)


      }))

      it(\` when called |
      and raw === true |
      does the required action \`,fakeAsync(()=>{
       // arrange
       ENV.${serviceString.camelCase(false)}.${serviceMethodString.orig}.automate = false
      // @ts-ignore
       cpnt.http.post = ()=>of(new ${serviceMethodString.capitalize(false)}APIResponseBody())
       spyOn(cpnt.http,"post").and.callThrough()
       let uiBody = new ${serviceMethodString.capitalize(false)}UIRequestBody()




      let successFn = (val?)=>{
        // assert
        discardPeriodicTasks()
      }
       // act

      cpnt.${serviceMethodString.orig}(uiBody,true)
      .subscribe({next:successFn,complete:successFn})
      expect(${serviceMethodString.orig}.${serviceMethodString.orig}Load).toHaveBeenCalledWith(uiBody)
      flush()
      expect(cpnt.http.post)
      .toHaveBeenCalledWith(ENV.${serviceString.camelCase(false)}.${serviceMethodString.orig}.url(),${serviceMethodString.orig}LoadResult)


      }))
    })

    describe("when ${serviceMethodString.orig}.automate === true",()=>{
      it(\` when called |
      under normal conditions |
      does the required action \`,fakeAsync(()=>{
       // arrange
       ENV.${serviceString.camelCase(false)}.${serviceMethodString.orig}.automate = true
        let sucessFn  = (val?)=>{
          // assert
          expect(val).toBeInstanceOf(${serviceMethodString.capitalize(false)}UIResponseBody)
          discardPeriodicTasks()
        }
       // act
      cpnt.${serviceMethodString.orig}()
      .subscribe({next:sucessFn})
      flush()

      }))
    })

  })`;
    let spacingPrefix = "\n  ";
    let insertPos = startNode.end;
    (0, utils_1.prepareForUpdate)(snippet, targetFilePath, tree, 
    // @ts-ignore
    insertPos, spacingPrefix);
    return sourceFile;
}
//# sourceMappingURL=index.bak.js.map